pub enum Item {
    Do,
    Dont,
    Mul(u32),
}

peg::parser! {
    grammar mul() for str {
        rule number() -> u32
            = n:$(['0'..='9']+) {? n.parse().or(Err("u32")) }

        rule mul_fn() -> Item
            = "mul(" a:number() "," b:number() ")" { Item::Mul(a * b) }

        rule do_fn() -> Item
            = "do()" { Item::Do }

        rule dont_fn() -> Item
            = "don't()" { Item::Dont }

        rule step() -> Option<Item>
            = n:mul_fn() { Some(n) }
            / n:do_fn() { Some(n) }
            / n:dont_fn() { Some(n) }
            / [_] { None }

        pub rule program() -> Vec<Item>
            = x:step()+ { x.into_iter().flatten().collect() }
    }
}

pub fn parse(a: &str) -> Vec<Item> {
    mul::program(a).unwrap()
}
